@@ -22,8 +22,12 @@
 from nuitka import Options
 from nuitka.PythonVersions import python_version
 
-from .CodeHelpers import generateChildExpressionsCode, generateExpressionCode
-from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode, getReleaseCodes
+from .CodeHelpers import (
+    generateChildExpressionsCode,
+    generateExpressionCode,
+    withObjectCodeTemporaryAssignment
+)
+from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode
 
 
 def generateBuiltinDictCode(to_name, expression, emit, context):
@@ -40,75 +44,65 @@ def generateBuiltinDictCode(to_name, expression, emit, context):
     else:
         seq_name = None
 
-    if expression.getNamedArgumentPairs():
-        # If there is no sequence to mix in, then directly generate
-        # into to_name.
+    with withObjectCodeTemporaryAssignment(to_name, "dict_result", expression, emit, context) \
+      as value_name:
 
-        if seq_name is None:
-            getDictionaryCreationCode(
-                to_name = to_name,
-                pairs   = expression.getNamedArgumentPairs(),
-                emit    = emit,
-                context = context
-            )
+        if expression.getNamedArgumentPairs():
+            # If there is no sequence to mix in, then directly generate
+            # into to_name.
 
-            dict_name = None
+            if seq_name is None:
+                _getDictionaryCreationCode(
+                    to_name = value_name,
+                    pairs   = expression.getNamedArgumentPairs(),
+                    emit    = emit,
+                    context = context
+                )
+
+                dict_name = None
+            else:
+                dict_name = context.allocateTempName("dict_arg")
+
+                _getDictionaryCreationCode(
+                    to_name = dict_name,
+                    pairs   = expression.getNamedArgumentPairs(),
+                    emit    = emit,
+                    context = context
+                )
         else:
-            dict_name = context.allocateTempName("dict_arg")
+            dict_name = None
 
-            getDictionaryCreationCode(
-                to_name = dict_name,
-                pairs   = expression.getNamedArgumentPairs(),
-                emit    = emit,
-                context = context
+        if seq_name is not None:
+            emit(
+                "%s = TO_DICT( %s, %s );" % (
+                    value_name,
+                    seq_name,
+                    "NULL" if dict_name is None else dict_name
+                )
             )
-    else:
-        dict_name = None
 
-    if seq_name is not None:
-        emit(
-            "%s = TO_DICT( %s, %s );" % (
-                to_name,
-                seq_name,
-                "NULL" if dict_name is None else dict_name
+            getErrorExitCode(
+                check_name    = value_name,
+                release_names = (seq_name, dict_name),
+                emit          = emit,
+                context       = context
             )
-        )
 
-        getReleaseCodes(
-            release_names = (seq_name, dict_name),
-            emit          = emit,
-            context       = context
-        )
-
-        getErrorExitCode(
-            check_name = to_name,
-            emit       = emit,
-            context    = context
-        )
-
-        context.addCleanupTempName(to_name)
+            context.addCleanupTempName(value_name)
 
 
 def generateDictionaryCreationCode(to_name, expression, emit, context):
-    getDictionaryCreationCode(
-        to_name = to_name,
-        pairs   = expression.getPairs(),
-        emit    = emit,
-        context = context
-    )
-
-
-def getDictionaryCreationCode(to_name, pairs, emit, context):
-
-    emit(
-        "%s = _PyDict_NewPresized( %d );" % (
-            to_name,
-            len(pairs)
+    with withObjectCodeTemporaryAssignment(to_name, "dict_result", expression, emit, context) \
+      as value_name:
+        _getDictionaryCreationCode(
+            to_name = value_name,
+            pairs   = expression.getPairs(),
+            emit    = emit,
+            context = context
         )
-    )
 
-    context.addCleanupTempName(to_name)
 
+def _getDictionaryCreationCode(to_name, pairs, emit, context):
     def generateValueCode(dict_value_name, pair):
         generateExpressionCode(
             to_name    = dict_value_name,
@@ -125,13 +119,14 @@ def getDictionaryCreationCode(to_name, pairs, emit, context):
             context    = context
         )
 
+    assert pairs
 
-    # Strange as it is, CPython evaluates the key/value pairs strictly in order,
-    # but for each pair, the value first.
-    for pair in pairs:
+    for count, pair in enumerate(pairs):
         dict_key_name = context.allocateTempName("dict_key")
         dict_value_name = context.allocateTempName("dict_value")
 
+        # Strange as it is, CPython 3.5 and before evaluated the key/value pairs
+        # strictly in order, but for each pair, the value first.
         if python_version < 350:
             generateValueCode(dict_value_name, pair)
             generateKeyCode(dict_key_name, pair)
@@ -139,8 +134,17 @@ def getDictionaryCreationCode(to_name, pairs, emit, context):
             generateKeyCode(dict_key_name, pair)
             generateValueCode(dict_value_name, pair)
 
-        needs_check = not pair.getKey().isKnownToBeHashable()
+        if count == 0:
+            emit(
+                "%s = _PyDict_NewPresized( %d );" % (
+                    to_name,
+                    len(pairs)
+                )
+            )
 
+            context.addCleanupTempName(to_name)
+
+        needs_check = not pair.getKey().isKnownToBeHashable()
         res_name = context.getIntResName()
 
         emit(
@@ -186,6 +190,8 @@ def generateDictOperationUpdateCode(statement, emit, context):
         context    = context
     )
 
+    old_source_ref = context.setCurrentSourceCodeReference(statement.getSourceReference())
+
     res_name = context.getIntResName()
 
     emit("assert( PyDict_Check( %s ) );" % dict_arg_name)
@@ -197,18 +203,15 @@ def generateDictOperationUpdateCode(statement, emit, context):
         )
     )
 
-    getReleaseCodes(
+    getErrorExitBoolCode(
+        condition     = "%s != 0" % res_name,
         release_names = (dict_arg_name, value_arg_name),
+        needs_check   = statement.mayRaiseException(BaseException),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitBoolCode(
-        condition   = "%s != 0" % res_name,
-        needs_check = statement.mayRaiseException(BaseException),
-        emit        = emit,
-        context     = context
-    )
+    old_source_ref = context.setCurrentSourceCodeReference(old_source_ref)
 
 
 def generateDictOperationGetCode(to_name, expression, emit, context):
@@ -218,28 +221,25 @@ def generateDictOperationGetCode(to_name, expression, emit, context):
         context    = context
     )
 
-    emit(
-        "%s = DICT_GET_ITEM( %s, %s );" % (
-            to_name,
-            dict_name,
-            key_name
+    with withObjectCodeTemporaryAssignment(to_name, "dict_value", expression, emit, context) \
+      as value_name:
+        emit(
+            "%s = DICT_GET_ITEM( %s, %s );" % (
+                value_name,
+                dict_name,
+                key_name
+            )
         )
-    )
 
-    getReleaseCodes(
-        release_names = (dict_name, key_name),
-        emit          = emit,
-        context       = context
-    )
+        getErrorExitCode(
+            check_name    = value_name,
+            release_names = (dict_name, key_name),
+            needs_check   = expression.mayRaiseException(BaseException),
+            emit          = emit,
+            context       = context
+        )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        needs_check = expression.mayRaiseException(BaseException),
-        emit        = emit,
-        context     = context
-    )
-
-    context.addCleanupTempName(to_name)
+        context.addCleanupTempName(value_name)
 
 
 def generateDictOperationInCode(to_name, expression, emit, context):
@@ -261,26 +261,21 @@ def generateDictOperationInCode(to_name, expression, emit, context):
         )
     )
 
-
-    getReleaseCodes(
+    getErrorExitBoolCode(
+        condition     = "%s == -1" % res_name,
         release_names = (dict_name, key_name),
+        needs_check   = expression.mayRaiseException(BaseException),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitBoolCode(
-        condition   = "%s == -1" % res_name,
-        needs_check = expression.mayRaiseException(BaseException),
-        emit        = emit,
-        context     = context
-    )
-
-    emit(
-        "%s = BOOL_FROM( %s == %s );" % (
-            to_name,
+    to_name.getCType().emitAssignmentCodeFromBoolCondition(
+        to_name   = to_name,
+        condition = "%s %s 0" % (
             res_name,
-            '1' if not inverted else '0'
-        )
+            "==" if inverted else "!="
+        ),
+        emit      = emit
     )
 
 
@@ -321,19 +316,14 @@ def generateDictOperationSetCode(statement, emit, context):
         )
     )
 
-    getReleaseCodes(
+    getErrorExitBoolCode(
+        condition     = "%s != 0" % res_name,
         release_names = (value_arg_name, dict_arg_name, key_arg_name),
         emit          = emit,
+        needs_check   = not statement.getKey().isKnownToBeHashable(),
         context       = context
     )
 
-    getErrorExitBoolCode(
-        condition   = "%s != 0" % res_name,
-        emit        = emit,
-        needs_check = not statement.getKey().isKnownToBeHashable(),
-        context     = context
-    )
-
 
 def generateDictOperationRemoveCode(statement, emit, context):
     dict_arg_name = context.allocateTempName("dictdel_dict", unique = True)
@@ -368,17 +358,12 @@ def generateDictOperationRemoveCode(statement, emit, context):
         )
     )
 
-    getReleaseCodes(
+    getErrorExitBoolCode(
+        condition     = "%s == false" % res_name,
         release_names = (dict_arg_name, key_arg_name),
+        needs_check   = statement.mayRaiseException(BaseException),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitBoolCode(
-        condition   = "%s == false" % res_name,
-        needs_check = statement.mayRaiseException(BaseException),
-        emit        = emit,
-        context     = context
-    )
-
     context.setCurrentSourceCodeReference(old_source_ref)
