@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,6 +28,8 @@ module.
 
 import os
 
+from nuitka.utils.Utils import getOS
+
 
 def getStandardLibraryPaths():
     """ Get the standard library paths.
@@ -50,7 +52,7 @@ def getStandardLibraryPaths():
         # Happens for virtualenv situation, some modules will come from the link
         # this points to.
         if os.path.islink(os_filename):
-            os_filename = os.readlink(os_filename)
+            os_filename = os.readlink(os_filename)  # @UndefinedVariable
             stdlib_paths.add(
                 os.path.normcase(
                     os.path.dirname(os_filename)
@@ -108,6 +110,12 @@ def getStandardLibraryPaths():
             if os.path.isdir(candidate):
                 stdlib_paths.add(candidate)
 
+        if getOS() == "Windows":
+            import _ctypes
+            stdlib_paths.add(
+                os.path.dirname(_ctypes.__file__)
+            )
+
         getStandardLibraryPaths.result = [
             os.path.normcase(stdlib_path)
             for stdlib_path in
@@ -136,4 +144,5 @@ def isStandardLibraryPath(path):
     for candidate in getStandardLibraryPaths():
         if path.startswith(candidate):
             return True
+
     return False
