@@ -582,6 +582,13 @@ class ExpressionBase(NodeBase):
 
         return statement, None, None
 
+    def computeExpressionBool(self, trace_collection):
+        if not self.mayRaiseException(BaseException) and \
+           self.mayRaiseExceptionBool(BaseException):
+            trace_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
     def onContentEscapes(self, trace_collection):
         pass
 
@@ -950,6 +957,21 @@ Predicted '%s' on compiled time constant values.""" % in_node.comparator
 
         return in_node, None, None
 
+    def computeExpressionBool(self, trace_collection):
+        constant = self.getCompileTimeConstant()
+
+        if type(constant) is not bool:
+            self.parent.replaceChild(
+                self,
+                makeConstantReplacementNode(bool(constant), self)
+            )
+
+            trace_collection.signalChange(
+                tags       = "new_constant",
+                source_ref = self.source_ref,
+                message    = "Predicted compile time constant true value."
+            )
+
 
 class ExpressionChildrenHavingBase(ChildrenHavingMixin, ExpressionBase):
     def __init__(self, values, source_ref):
@@ -1098,7 +1120,7 @@ class ExpressionChildHavingBase(ExpressionBase):
 
         attr_name = "subnode_" + name
 
-        # Determine old value, and inform it about loosing its parent.
+        # Determine old value, and inform it about losing its parent.
         old_value = getattr(self, attr_name)
 
         assert old_value is not value, value
