@@ -30,16 +30,23 @@ from nuitka.utils.InstanceCounters import counted_del, counted_init
 _future_division_default = python_version >= 300
 _future_absolute_import_default = python_version >= 300
 _future_generator_stop_default = python_version >= 370
+_future_annotations_default = python_version >= 400
+
 
 class FutureSpec(object):
+    __slots__ = ("future_division", "unicode_literals", "absolute_import",
+                 "future_print", "barry_bdfl", "generator_stop",
+                 "future_annotations")
+
     @counted_init
     def __init__(self):
-        self.future_division  = _future_division_default
-        self.unicode_literals = False
-        self.absolute_import  = _future_absolute_import_default
-        self.future_print     = False
-        self.barry_bdfl       = False
-        self.generator_stop   = _future_generator_stop_default
+        self.future_division    = _future_division_default
+        self.unicode_literals   = False
+        self.absolute_import    = _future_absolute_import_default
+        self.future_print       = False
+        self.barry_bdfl         = False
+        self.generator_stop     = _future_generator_stop_default
+        self.future_annotations = _future_annotations_default
 
     __del__ = counted_del()
 
@@ -49,11 +56,13 @@ class FutureSpec(object):
     def clone(self):
         result = FutureSpec()
 
-        result.future_division   = self.future_division
-        result.unicode_literals  = self.unicode_literals
-        result.absolute_import   = self.absolute_import
-        result.future_print      = self.future_print
-        result.barry_bdfl        = self.barry_bdfl
+        result.future_division    = self.future_division
+        result.unicode_literals   = self.unicode_literals
+        result.absolute_import    = self.absolute_import
+        result.future_print       = self.future_print
+        result.barry_bdfl         = self.barry_bdfl
+        result.generator_stop     = self.generator_stop
+        result.future_annotations = result.future_annotations
 
         return result
 
@@ -87,6 +96,12 @@ class FutureSpec(object):
     def isGeneratorStop(self):
         return self.generator_stop
 
+    def enableFutureAnnotations(self):
+        self.future_annotations = True
+
+    def isFutureAnnotations(self):
+        return self.future_annotations
+
     def asFlags(self):
         """ Create a list of C identifiers to represent the flag values.
 
@@ -114,6 +129,9 @@ class FutureSpec(object):
         if python_version >= 350 and python_version < 370 and self.generator_stop:
             result.append("CO_FUTURE_GENERATOR_STOP")
 
+        if python_version >= 370 and self.future_annotations:
+            result.append("CO_FUTURE_ANNOTATIONS")
+
         return tuple(result)
 
 
