@@ -132,6 +132,13 @@ def makeRaiseExceptionExpressionFromTemplate(exception_type, template,
     from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
     from .OperatorNodes import makeBinaryOperationNode
     from .ConstantRefNodes import makeConstantRefNode
+    from .ContainerMakingNodes import ExpressionMakeTuple
+
+    if type(template_args) is tuple:
+        template_args = ExpressionMakeTuple(
+            elements   = template_args,
+            source_ref = source_ref
+        )
 
     return ExpressionRaiseException(
         exception_type  = ExpressionBuiltinExceptionRef(
@@ -151,6 +158,7 @@ def makeRaiseExceptionExpressionFromTemplate(exception_type, template,
         source_ref      = source_ref
     )
 
+
 def makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue(template,
                                                                operation,
                                                                original_node,
@@ -385,54 +393,6 @@ def makeStatementOnlyNodesFromExpressions(expressions):
         )
 
 
-def makeComparisonNode(left, right, comparator, source_ref):
-    from .ComparisonNodes import (
-        ExpressionComparison,
-        ExpressionComparisonIs,
-        ExpressionComparisonIsNOT,
-        ExpressionComparisonIn,
-        ExpressionComparisonNOTIn
-    )
-
-    if comparator == "Is":
-        result = ExpressionComparisonIs(
-            left       = left,
-            right      = right,
-            source_ref = source_ref
-        )
-    elif comparator == "IsNot":
-        result = ExpressionComparisonIsNOT(
-            left       = left,
-            right      = right,
-            source_ref = source_ref
-        )
-    elif comparator == "In":
-        result = ExpressionComparisonIn(
-            left       = left,
-            right      = right,
-            source_ref = source_ref
-        )
-    elif comparator == "NotIn":
-        result = ExpressionComparisonNOTIn(
-            left       = left,
-            right      = right,
-            source_ref = source_ref
-        )
-    else:
-        result = ExpressionComparison(
-            left       = left,
-            right      = right,
-            comparator = comparator,
-            source_ref = source_ref
-        )
-
-    result.setCompatibleSourceReference(
-        source_ref = right.getCompatibleSourceReference()
-    )
-
-    return result
-
-
 def makeVariableRefNode(variable, source_ref):
     if variable.isTempVariable():
         from .VariableRefNodes import ExpressionTempVariableRef
@@ -467,20 +427,24 @@ def makeExpressionBuiltinLocals(provider, source_ref):
             ExpressionBuiltinLocalsUpdated
         )
 
-        # TODO: make this dependent on getLocalsScope not being None.
         if provider.isExpressionClassBody():
             return ExpressionBuiltinLocalsRef(
-                locals_scope = provider.getLocalsScope(),
+                locals_scope = provider.getFunctionLocalsScope(),
                 source_ref   = source_ref
             )
         elif python_version >= 300 or provider.isUnoptimized():
-            assert provider.getLocalsScope(), provider
+            assert provider.getFunctionLocalsScope(), provider
 
             return ExpressionBuiltinLocalsUpdated(
-                locals_scope = provider.getLocalsScope(),
+                locals_scope = provider.getFunctionLocalsScope(),
                 source_ref   = source_ref
             )
         else:
+            # TODO: Make this not true, there ought to be always a locals
+            # scope.
+            assert not provider.getFunctionLocalsScope(), provider
+
             return ExpressionBuiltinLocalsCopy(
-                source_ref = source_ref
+                locals_scope = provider.getFunctionLocalsScope(),
+                source_ref   = source_ref
             )
